/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.mapper.rdb.adapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;

/**
 * This class is a sql adapter for the SapDB relational database.
 * SapDB 7.4 beta
 * result of the jdbc driver test:
 * ===============================
 * FAIL: BIT: create table: [-3002] (at 31): Invalid datatype
 * FAIL: TINYINT: create table: [-4006] (at 28): Unknown domain name:TINYINT
 * FAIL: SMALLINT: select value 0 with getObject
 * FAIL: SMALLINT: select value 1 with getObject
 * FAIL: SMALLINT: select value 254 with getObject
 * FAIL: INTEGER: select value 0 with getObject
 * FAIL: INTEGER: select value 1 with getObject
 * FAIL: INTEGER: select value 254 with getObject
 * FAIL: BIGINT: create table: [-4006] (at 28): Unknown domain name:BIGINT
 * FAIL: REAL: select value 0.0 with getObject
 * FAIL: REAL: select value 1.0 with getObject
 * FAIL: REAL: select value 254.0 with getObject
 * FAIL: DOUBLE: create table: [-3002] (at 30): Invalid datatype
 *
 * @author S.Chassande-Barrioz
 */
public class SapdbAdapter extends BasicRdbAdapter {

	public SapdbAdapter() {
		super("sapdb");
	}

	public void writeTableAlias(String alias, StringBuffer sb) {
		sb.append(" ");
		sb.append(alias);
	}

	public String getNextValInSequence(String seqName) {
		return "select " + seqName + " .nextval from dual";
	}
	
	public void getManyNextValInSequence(Connection conn, String seqName, int numberOfNextVal) 
	throws RdbAdapterException {
		StringBuffer sb = new StringBuffer();
		sb.append("select ");
		for (int i = 0; i < numberOfNextVal -1; i++) {
			sb.append( seqName + ".nextval, ");
		}
		sb.append(seqName + ".nextval from dual");
		PreparedStatement ps = null;
		try {
			ps = conn.prepareStatement(sb.toString());
			ps.executeQuery();
		} catch (SQLException e) {
			throw new RdbAdapterException(e, "Impossible to allocate " + numberOfNextVal + " identifier"
				+ " on the sequence: " + seqName);
		} finally {
			try {
				if (ps != null) {
					ps.close();
				}
			} catch (SQLException e) {
			}
		}
	}
}
