/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.mapper.rdb.adapter;

import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.Types;

/**
 * This class is a sql adapter for the SQL Server relational database.
 *
 * @author P. Dechamboux
 */
public class SqlserverAdapter extends BasicRdbAdapter {

	public SqlserverAdapter() {
		super("sqlserver");
	}

	public String getSqlType(int typeCode, boolean usedInPK, int size, int scale) throws RdbAdapterException {
		switch (typeCode) {
		case TYPECODE_BYTE:
		case TYPECODE_OBJBYTE:
			return "SMALLINT";
		case TYPECODE_BYTEARRAY:
			if ((size != NOSIZE) && (size <= 8000)) {
				return "VARBINARY(" + size + ")";
			}
			return "IMAGE";
		case TYPECODE_CHARARRAY:
			if ((size != NOSIZE) && (size <= 8000)) {
				return "VARCHAR(" + size + ")";
			}
			return "TEXT";
		case TYPECODE_FLOAT:
		case TYPECODE_OBJFLOAT:
			return "FLOAT(24)";
		case TYPECODE_DOUBLE:
		case TYPECODE_OBJDOUBLE:
			return "FLOAT(53)";
		case TYPECODE_DATE:
			return "DATETIME";
		case TYPECODE_BIGINTEGER:
		case TYPECODE_BIGDECIMAL:
			return "MONEY";
		default:
			return super.getSqlType(typeCode, usedInPK, size, scale);
		}
	}

	public char[] getCharArray(ResultSet rs, int idx, char[] nullValue)
			throws SQLException {
		String res = rs.getString(idx);
		try {
			return (rs.wasNull() || res == null ? nullValue : getCharArray(res));
		} catch (SQLException e) {
			throw e;
		} catch (Exception e) {
			throw new SQLException(e.getMessage());
		}
	}

	public void setObyte(PreparedStatement ps, int idx, Byte o)
			throws SQLException {
		ps.setObject(idx, o, Types.SMALLINT);
	}

	public void setOchar(PreparedStatement ps, int idx, Character o)
			throws SQLException {
		ps.setString(idx, o.toString());
	}

	public void setCharArray(PreparedStatement ps, int idx, char[] o)
			throws SQLException {
		ps.setString(idx, new String(o));
	}

	public String getCreateSequence(String seqName)
	    throws UnsupportedOperationException {
		throw new UnsupportedOperationException("SQL Server does not support SEQUENCE") ;
	}

	public String getCreateSequence(String seqName, Integer startid, Integer inc, Integer cache) {
		throw new UnsupportedOperationException("SQL Server does not support SEQUENCE") ;
	}

	public String getDropSequence(String seqName)
    	throws UnsupportedOperationException {
		throw new UnsupportedOperationException("SQL Server does not support SEQUENCE") ;
	}
	
	public String getLengthOperator() throws RdbAdapterException {
		return "len";
	}
}
